﻿' EXAMPLE - SQL12 for Visual Basic using ADO.NET
' This example shows multi-user access to a database using separate threads
' and a remote tfserver.  The connections will use TCP/IP to communicate
' with the tfserver to do their work.  After starting the tfserver, two
' writer threads are created.  The first begins inserting values into the
' table.  The second waits until the table contains more than 100 rows and
' then begins deleting the oldest.  A reader thread is also created that
' prints out the current oldest and newest entries every second.  Each
' thread uses regular locking.
' 
' The following methods are illustrated:
'     Raima.Rdm.RdmConnection:
'         BeginTransaction() Method
'         ChangeDatabase(string) Method
'         Close() Method
'         CreateCommand() Method
'         Open() Method
'         RdmConnection() Constructor
'     Raima.Rdm.RdmCommand:
'         CommandText Property
'         ExecuteNonQuery() Method
'         ExecuteReader() Method
'         Parameters Property
'     Raima.Rdm.RdmDataReader:
'         Item[int] Property
'         Read() Method
'     Raima.Rdm.RdmParameterCollection
'         Add(string, RdmType)
'         Item[string] Property
' 
' They are fully document in http://docs.raima.com/rdm/.
' 
' For simplicity this example does not catch any thrown execptions
' but good programming practices would dictate that this occur in
' a full application.

' Raima.Rdm is the system module that contains the ADO.NET
' data provider for RDM
Imports Rdm = Raima.Rdm

Module SQL12
    Dim shutdown As Boolean

    Sub DropDatabase(conn As Rdm.RdmConnection, name As String)
        Dim cmd As Rdm.RdmCommand
        cmd = conn.CreateCommand()

        ' Attempt to drop this database.  If the database does not already
        ' exist this will fail and we simply ignore the failure for this
        ' sample.
        Try
            cmd.CommandText = "DROP DATABASE " + name
            cmd.ExecuteNonQuery()
        Catch ex As Exception
        End Try

    End Sub

    Sub CreateThisDatabase(conn As Rdm.RdmConnection, name As String, cmds() As String)
        Dim cmd As Rdm.RdmCommand
        cmd = conn.CreateCommand()

        cmd.CommandText = "CREATE DATABASE " + name
        cmd.ExecuteNonQuery()

        For Each c In cmds
            cmd.CommandText = c
            cmd.ExecuteNonQuery()
        Next

        cmd.CommandText = "COMMIT"
        cmd.ExecuteNonQuery()

        cmd.CommandText = "CLOSE " + name
        cmd.ExecuteNonQuery()
    End Sub

    Sub CreateDatabases(conn As Rdm.RdmConnection)
        ' Drop any existing copy of the databases.
        DropDatabase(conn, "SQL12")

        ' Attempt to create the database
        Dim cmds(0) As String
        cmds(0) = "CREATE TABLE Table1 (id INTEGER PRIMARY KEY)"
        CreateThisDatabase(conn, "SQL12", cmds)
    End Sub

    Sub InserterThreadProc()
        ' Create a connection for this thread
        Dim conn As Rdm.RdmConnection
        conn = New Rdm.RdmConnection("host=local;database=SQL12;openmode=s;tfstype=tfsr")
        conn.Open()

        ' Create a command to do the insert
        Dim cmd As Rdm.RdmCommand
        cmd = conn.CreateCommand()
        cmd.CommandText = "INSERT INTO Table1 VALUES :id"
        cmd.Parameters.Add("id", Rdm.RdmType.Int32)

        ' Loop until a shutdown is requested
        Dim ii As Int32
        While shutdown = False
            'Insert the next value
            cmd.Parameters("id").Value = ii
            cmd.ExecuteNonQuery()
            ii = ii + 1
        End While

        ' When a shutdown is requested, close the connection and exit
        conn.Close()
    End Sub

    Sub DeleterThreadProc()
        ' Create a connection for this thread
        Dim conn As Rdm.RdmConnection
        conn = New Rdm.RdmConnection("host=local;database=SQL12;openmode=s;tfstype=tfsr")
        conn.Open()

        ' Create a command to check the number of rows, one to find the
        ' 'smallest' row, and one to delete
        Dim chkCmd, minCmd, delCmd As Rdm.RdmCommand
        chkCmd = conn.CreateCommand()
        minCmd = conn.CreateCommand()
        delCmd = conn.CreateCommand()

        chkCmd.CommandText = "SELECT COUNT(id) FROM Table1"
        minCmd.CommandText = "SELECT MIN(id) FROM Table1"
        delCmd.CommandText = "DELETE FROM Table1 WHERE id = :minid"
        Dim parm As Rdm.RdmParameter
        parm = delCmd.Parameters.Add("minid", Rdm.RdmType.Int32)

        Dim count As Long
        Dim countRdr, minRdr As Rdm.RdmDataReader

        ' Loop until a shutdown is requested
        While shutdown = False
            ' Query the number of rows in the table
            countRdr = chkCmd.ExecuteReader()
            If (countRdr.Read()) Then
                count = countRdr(0)

                ' There should be only one row, so this closes the reader
                countRdr.Read()

                If count > 100 Then
                    ' If there are at least 100 rows, get the smallest and delete it
                    minRdr = minCmd.ExecuteReader()
                    If minRdr.Read() Then
                        parm.Value = Convert.ToInt32(minRdr(0))
                        delCmd.ExecuteNonQuery()

                        ' There should be only one row, so this closes the reader
                        minRdr.Read()
                    End If
                End If
            End If
        End While

        conn.Close()
    End Sub

    Sub ReaderThreadProc()
        ' Create a connection for this thread
        Dim conn As Rdm.RdmConnection
        conn = New Rdm.RdmConnection("host=local;database=SQL12;openmode=s;tfstype=tfsr")
        conn.Open()

        ' Create a command to do the read
        Dim cmd As Rdm.RdmCommand
        cmd = conn.CreateCommand()
        cmd.CommandText = "SELECT MIN(id), MAX(id) from Table1"

        Dim rdr As Rdm.RdmDataReader

        ' Loop until a shutdown is requested
        While shutdown = False
            ' Wait one second
            Threading.Thread.Sleep(1000)

            'Query the minimum and maximum values
            rdr = cmd.ExecuteReader()
            If rdr.Read() Then
                Console.WriteLine("Current minimum = " + Convert.ToString(rdr(0)) + " and current maximum = " + Convert.ToString(rdr(1)))
                rdr.Read()
            End If
        End While

        conn.Close()
    End Sub

    Sub Main()
        shutdown = False

        Console.WriteLine("Starting tfserver (and waiting 5 seconds for it to initialize)")

        ' Start the tfserver process to process 'remote' connections
        Dim tfserver As Process
        tfserver = New Process()
        tfserver.StartInfo.FileName = "tfserver.exe"
        tfserver.Start()

        ' Sleep to give the tfserver time to initialize and start listening
        Threading.Thread.Sleep(5000)

        ' Create the connection with a simple connection string
        Dim conn As Rdm.RdmConnection
        conn = New Rdm.RdmConnection("host=local;openmode=s;tfstype=tfsr")
        conn.Open()

        CreateDatabases(conn)

        ' Create three threads, one to insert, one to delete, and one to
        ' show the current contents
        Dim inserter, deleter, reader As Threading.Thread
        inserter = New Threading.Thread(New Threading.ThreadStart(AddressOf InserterThreadProc))
        deleter = New Threading.Thread(New Threading.ThreadStart(AddressOf DeleterThreadProc))
        reader = New Threading.Thread(New Threading.ThreadStart(AddressOf ReaderThreadProc))

        Console.WriteLine("Starting threads")
        Console.WriteLine("   Press <Enter> when you are ready to terminate them")

        ' Start each of the threads
        inserter.Start()
        deleter.Start()
        reader.Start()

        ' Wait for the user to hit <Enter>
        Console.ReadLine()

        ' Indicate to the threads that they should shut down
        shutdown = True

        ' Wait for each thread to finish
        inserter.Join()
        deleter.Join()
        reader.Join()

        Console.WriteLine("All threads have terminated")

        conn.Close()

        ' Stop the tfserver
        tfserver.CloseMainWindow()
        tfserver.WaitForExit()
        tfserver.Close()

        Console.WriteLine("Hit <Enter> to continue")
        Console.ReadLine()
    End Sub
End Module
